# Generated by `rjournal_pdf_article()` using `knitr::purl()`: do not edit by hand
# Please edit brolgar-paper.Rmd to modify this file

## ----setup, echo=FALSE, message=FALSE, warning=FALSE, comment = FALSE---------
knitr::opts_chunk$set(warning = FALSE, 
                      message = FALSE, 
                      echo = TRUE,
                      out.width = "95%",
                      retina = 3,
                      fig.height = 4,
                      fig.align = "center",
                      dpi = 300,
                      dev = "png")


## ----source-pkgs, echo = FALSE------------------------------------------------
#suppressMessages(source("packages.R"))
#suppressMessages(source("conflicts.R"))
library("tidyverse")
library("brolgar")
library("conflicted")
library("colorspace")
library("dotenv")
library("future")
library("fs")
library("mgcv")
library("stringr")
library("modelr")
library("tsibble")
library("tsibbletalk")
library("plotly")
library("patchwork")
library("gridExtra")
# remotes::install_github("jimjam-slam/stickylabeller")
library("stickylabeller")


## ----load, echo = FALSE-------------------------------------------------------
# tar_load(heights_gam)
# tar_load(model_time)
# tar_load(heights_brolgar)
# tar_load(paper_bib)
#heights_gam <- read_rds("rds/heights_gam.rds")
#heights_brolgar <- read_rds("rds/heights_brolgar.rds")
#model_time <- read_rds("rds/model_time.rds")
load("heights_gam.rda")
load("heights_brolgar.rda")
load("model_time.rda")


## ----conflicts, echo=FALSE----------------------------------------------------
conflict_prefer("filter", "dplyr")
conflict_prefer("select", "dplyr")
conflict_prefer("pluck", "purrr")
conflict_prefer("map", "purrr")
conflict_prefer("pull", "dplyr")


## ----heights-sample-plot, fig.cap = "Example of longitudinal data: average height of men in Australia for 1900-1970. The height increase over time, and are measured at irregular intervals.", echo = FALSE, fig.height = 4, fig.width = 12----

heights_oz <- heights_brolgar %>% 
  filter(country == "Australia") %>% 
  slice_tail(n = 5) 

mytheme <- gridExtra::ttheme_default(
  core = list(padding = unit(c(5, 10), "mm"))
  )

tbl_heights_oz <- heights_oz %>% 
  mutate(height_cm = round(height_cm, 1)) %>% 
  tableGrob(theme = mytheme, 
            rows = NULL)

gg_heights_oz <- ggplot(heights_oz,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_point() + 
  geom_line()
# NOTE - add time points for each measurement?

grid.arrange(tbl_heights_oz,
             gg_heights_oz,
             nrow = 1,
             as.table = TRUE)


## ----spaghetti, fig.height = 4, fig.width = 12, message = FALSE, fig.cap = "The full dataset shown as a spaghetti plot (A), with transparency (B), and with a linear model overlayed (C). It is still hard to see the individuals.", echo = FALSE, warning=FALSE----
spaghetti <- ggplot(heights_brolgar,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_line()

spaghetti_alpha <- ggplot(heights_brolgar,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_line(alpha = 0.3)

spaghetti_smooth <- spaghetti_alpha + geom_smooth(method = "lm", 
                                                  aes(group = NULL))

spaghetti + spaghetti_alpha + spaghetti_smooth + 
  plot_annotation(tag_levels = "A")


## ----define-heights, eval = FALSE---------------------------------------------
#> heights_brolgar <- as_tsibble(heights_brolgar,
#>                       index = year,
#>                       key = country,
#>                       regular = FALSE)
#> 


## ----heights-summary----------------------------------------------------------
heights_min <- features(.tbl = heights_brolgar, 
                        .var = height_cm, 
                        features = c(min = min))

heights_min


## ----heights-example-three-features-------------------------------------------
heights_three <- heights_brolgar %>%
  features(height_cm, c(
    min = min,
    median = median,
    max = max
  ))

heights_three


## ----feature-min-med-max, fig.cap = "Three plots showing the distribution of minimum, median, and maximum values of height in centimeters. Part A shows just the distribution of minimum, part B shows the distribution of minimum, median, and maximum, and part C shows these three values plotted together as a line graph. We see that there is overlap amongst all three statistics. That is, some countries minimum heights are taller than some countries maximum heights.", fig.height = 3, fig.width = 9, echo = FALSE----
p1 <- ggplot(heights_min,
       aes(x = min)) + 
  geom_density(fill = qualitative_hcl(n = 3, palette = "Dark3")[1]) 

p2 <- heights_three %>% 
  pivot_longer(cols = min:max,
               names_to = "feature",
               values_to = "value") %>% 
  ggplot(aes(x = value,
             fill = feature)) + 
  geom_density(alpha = 0.5) +
  labs(x = "Value",
       y = "Density",
       fill = "Feature") + 
  scale_fill_discrete_qualitative(palette = "Dark 3") +
  # inset the legend?
  theme(legend.position = "none")


p3 <- 
heights_three %>% 
  pivot_longer(cols = min:max,
               names_to = "feature",
               values_to = "value") %>% 
  mutate(feature = factor(feature,
                          levels = c("min",
                                     "median",
                                     "max"))) %>% 
ggplot(aes(x = feature,
           y = value,
           group = country)) + 
  geom_line(alpha = 0.6)

p1 + p2 + p3 + plot_annotation(tag_levels = "A") & theme(aspect.ratio = 1)


## ----features-fivenum, echo = TRUE--------------------------------------------
heights_five <- heights_brolgar %>%
  features(height_cm, feat_five_num)

heights_five


## ----show-features-set--------------------------------------------------------
library(fabletools)
feat_set_brolgar <- feature_set(pkgs = "brolgar")
length(feat_set_brolgar)


## ----run-features-set, message = FALSE----------------------------------------
heights_brolgar %>%
  features(height_cm, feat_set_brolgar)


## ----create-three, echo = TRUE------------------------------------------------

second <- function(x) nth(x, n = 2)
third <- function(x) nth(x, n = 3)

feat_first_three <- c(first = first,
                      second = second,
                      third = third)



## ----demo-feat-three, echo = TRUE---------------------------------------------
heights_brolgar %>%
  features(height_cm, feat_first_three)


## ----demo-feat-five-num, eval = FALSE, echo = TRUE----------------------------
#> feat_five_num <- function(x, ...) {
#>   c(
#>     min = b_min(x, ...),
#>     q25 = b_q25(x, ...),
#>     med = b_median(x, ...),
#>     q75 = b_q75(x, ...),
#>     max = b_max(x, ...)
#>   )
#> }


## ----facet-sample, fig.cap = "Twelve facets with three keys per facet shown. This allows us to quickly view a random sample of the data."----
ggplot(heights_brolgar,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_line() + 
  facet_sample() + 
  scale_x_continuous(breaks = c(1750, 1850, 1950))


## ----facet-strata, echo = TRUE, fig.cap = "All of the data is shown by spreading out each key across twelve facets. Each key is only shown once, and is randomly allocated to a facet."----
ggplot(heights_brolgar,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_line() + 
  facet_strata() +
  scale_x_continuous(breaks = c(1750, 1850, 1950))


## ----facet-year-along, echo = TRUE, fig.cap = "Displaying all the data across twelve facets. Instead of each key being randomly in a facet, each facet displays a specified range of values of year. In this case, the top left facet shows the keys with the earliest starting year, and the bottom right shows the facet with the latest starting year."----
ggplot(heights_brolgar,
       aes(x = year,
           y = height_cm,
           group = country)) + 
  geom_line() + 
  facet_strata(along = -year) + 
  scale_x_continuous(breaks = c(1750, 1850, 1950))


## ----index-summary-regular----------------------------------------------------
index_summary(heights_brolgar)
index_regular(heights_brolgar)


## -----------------------------------------------------------------------------
heights_brolgar %>% features(year, n_obs)


## -----------------------------------------------------------------------------
heights_brolgar %>% features(year, n_obs) %>% count(n_obs)


## ----overwrite-heights, eval = FALSE------------------------------------------
#> heights_brolgar <- heights %>%
#>   add_n_obs() %>%
#>   filter(n_obs >= 5)


## ----feature-first------------------------------------------------------------
heights_brolgar %>% 
  features(year, c(first = first))


## ----feature-first-gg, fig.cap = "Distribution of starting years of measurement. The data is already binned into 10 year blocks. Most of the years start between 1840 and 1900.", fig.height = 3, fig.width = 5, out.width = "60%"----
heights_brolgar %>% 
  features(year, c(first = first)) %>% 
  ggplot(aes(x = first)) +
  geom_bar()


## ----heights-diff-summary, message = FALSE, warning = FALSE-------------------
heights_diffs <- heights_brolgar %>% 
  features(year, feat_diff_summary)

heights_diffs


## ----diff-wide, message = FALSE, warning = FALSE------------------------------
heights_brolgar %>% 
  features(year, diff)


## ----heights-long-feat-diff, fig.cap = "Exploring the different summary statistics of the differences amongst the years. We learn that the smallest interval between measurements is 10 years, and the largest interval is between 10 and 125 years, and that most of the data is measured between 10 and 30 or so years.", echo = FALSE----
heights_diff_long <- heights_diffs %>% 
  pivot_longer(cols = -country,
               names_to = "feature",
               values_to = "value") %>% 
  mutate(feature = factor(feature,
                          levels = c("diff_min",
                                     "diff_q25",
                                     "diff_median",
                                     "diff_mean",
                                     "diff_q75",
                                     "diff_max",
                                     "diff_sd",
                                     "diff_var",
                                     "diff_iqr")))

ggplot(heights_diff_long,
       aes(x = value)) +
  geom_histogram(binwidth = 10,
                 colour = "gray") +
  facet_wrap(~feature,
             scales = "free")


## ----heights-max--------------------------------------------------------------
heights_max_in <- heights_brolgar %>% 
  features(height_cm, list(max = max,
                           increase = increasing))

heights_max_in


## ----heights-max-in-examine, fig.cap = "The different distributions of the features - A is depicting the distribution of maximum height, and B displays the number of countries that are always increasing (FALSE), and always increasing (TRUE). We note that the average maximum heights range from about 160cm to 185cm, with most being around 170cm. We also learn that the vast majority of countries are not always increasing in height through time.", echo = FALSE, fig.height = 3----

gg_max <- ggplot(heights_max_in,
       aes(x = max)) + 
  geom_histogram(binwidth = 1,
                 colour = "white") 

gg_inc <- ggplot(heights_max_in,
       aes(x = increase)) + 
  geom_bar()

gg_max + gg_inc + plot_annotation(tag_levels = "A")



## ----heights-max-in-full------------------------------------------------------
heights_max_in_full <- heights_max_in %>% 
  left_join(heights_brolgar,
            by = "country")

heights_max_in_full


## ----heights-increase---------------------------------------------------------
heights_increase <- heights_max_in_full %>% filter(increase)
heights_increase


## ----heights-top-5------------------------------------------------------------
heights_top <- heights_max_in_full %>% top_n(n = 1, wt = max)
heights_top


## ----gg-background, echo = FALSE----------------------------------------------
gg_height_bg <- ggplot(heights_max_in_full,
       aes(x = year,
           y = height_cm,
           group = country)) +
  geom_line(colour = "gray",
            alpha = 0.3) 


## ----gg-increase-max, echo = FALSE, fig.cap = "Plots of the data in the background, with the countries that always increase in height through time in A, and the country with the tallest people in B", fig.height = 3, eval=knitr::is_latex_output()----
(gg_height_bg + geom_line(data = heights_increase)) +
(gg_height_bg + geom_line(data = heights_top)) +
  plot_annotation(tag_levels = "A")


## ----interactive-gg-inc-max, echo = FALSE, fig.cap = "Interactive plot to explore longnostics of maximum height and slope from a simple linear fit, relative to the profiles. Click on either plot to select countries. For example, the country with the most negative slope, that is people are getting shorter is Eritrea.", layout = "l-body-outset", eval=knitr::is_html_output()----
#> slopes <- key_slope(brolgar::heights, height_cm ~ year)
#> heights_max_in_full <- heights_max_in_full %>%
#>   left_join(slopes) %>%
#>   rename(slope = .slope_year) %>%
#>   mutate(slope = round(slope, 3))
#> heights_max_in_full_ts <- heights_max_in_full %>%
#>   select(country, year, height_cm, max, slope) %>%
#>   as_tsibble(index = year, key = country) %>%
#>   as_shared_tsibble()
#> h1 <- ggplot(heights_max_in_full_ts, aes(x = slope,
#>                                       y = max,
#>                                       group = country)) +
#>   geom_point()
#> h2 <- ggplot(heights_max_in_full_ts, aes(x = year,
#>                                       y = height_cm,
#>                                       group = country)) +
#>   geom_line(alpha=0.2)
#> subplot(
#>     ggplotly(h1, tooltip = c("slope", "max", "country"),
#>              width = 100, height = 400),
#>     ggplotly(h2, tooltip = "country", width = 900, height = 400),
#>     nrows = 1, widths=c(0.4, 0.6)) %>%
#>   highlight(dynamic = TRUE)


## ----mgcv-fit, eval = FALSE---------------------------------------------------
#> heights_gam <- gam(
#>     height_cm ~ s(year0, by = country_fct) + country_fct,
#>     data = heights_brolgar,
#>     method = "REML"
#>   )


## ----tar-load-gam, echo = FALSE-----------------------------------------------
heights_gam <- heights_gam$model


## ----heights-lmer-------------------------------------------------------------
library(mgcv)
library(modelr)
heights_aug <- heights_brolgar %>%
  add_predictions(heights_gam, var = "pred") %>%
  add_residuals(heights_gam, var = "res") %>% 
  group_by_key() %>% 
  mutate(rss = sum(res^2)) %>% 
  ungroup()


## ----sample-n-keys-model, fig.cap = "Exploration of a random sample of the data. This shows the data points of 12 countries, with the model fit in blue."----
heights_aug %>% 
  sample_n_keys(12) %>% 
  ggplot(aes(x = year,
             y = pred,
             group = country)) + 
  geom_line(colour = "steelblue") +
  geom_point(aes(y = height_cm)) + 
  facet_wrap(~country)


## ----heights-aug-res-summary, fig.cap = "Five number summary of residual values from the model fit. The residuals are centered around zero with some variation.", echo = FALSE, out.width = "60%"----
heights_aug_res_summary <- 
tibble(names = names(summary(heights_aug$res)),
       values = as.numeric(round(summary(heights_aug$res),2))) %>% 
  slice(-4) %>% 
  mutate(y_pos = c(280, 275, 275, 250, 275))

gg_resid <- 
ggplot(heights_aug,
       aes(x = res)) + 
  geom_histogram(colour = "grey",
                 binwidth = 1) +
  # geom_density() +
  geom_rug(alpha = 0.2)

gg_resid + 
  geom_vline(xintercept = heights_aug_res_summary$values,
             colour = "salmon") + 
  ggrepel::geom_label_repel(data = heights_aug_res_summary,
             mapping = aes(label = names,
                           x = values,
                           y = y_pos))


## ----heights-aug-keys-near----------------------------------------------------
keys_near(heights_aug, var = rss)


## ----heights-keys-near-join---------------------------------------------------
heights_near_aug <- heights_aug %>% 
  keys_near(var = rss) %>% 
  left_join(heights_aug, 
            by = c("country"))


## ----heights-keys-near, fig.cap = "The keys nearest to the five number summary of the residual sums of squares. Moldova and Madagascar are well fit by the model, and are fit by a straight line. The remaining countries with poorer fit have greater variation in height. It is not clear how a better model fit could be achieved.", fig.height = 2.5, fig.width = 8----
ggplot(heights_near_aug,
       aes(x = year,
           y = pred,
           group = country,
           colour = country)) + 
  geom_line(colour = "orange") + 
  geom_point(aes(y = height_cm)) + 
  scale_x_continuous(breaks = c(1780, 1880, 1980)) +
  facet_wrap(~stat + country,
             labeller = label_glue("Country: {country} \nNearest to \n{stat} RSS"),
             nrow = 1) + 
  theme(legend.position = "none",
        aspect.ratio = 1)


## ----heights-keys-near-extra--------------------------------------------------
heights_near_aug_top_3 <- heights_aug %>% 
  distinct(country, rss) %>% 
  top_n(n = 3,
        wt = rss)

heights_near_aug_bottom_3 <- heights_aug %>% 
  distinct(country, rss) %>% 
  top_n(n = -3,
        wt = rss)

heights_near_top_bot_3 <- bind_rows(highest_3 = heights_near_aug_top_3,
                                    lowest_3 = heights_near_aug_bottom_3,
                                    .id = "rank") %>% 
  left_join(heights_aug,
            by = c("country", "rss"))


## ----heights-keys-near-fancy-label, fig.cap = "Figure of stereotypes for those keys with the three highest and lowest RSS values. Those that fit best tend to be linear, but those that fit worst have wider variation in heights.", echo = FALSE, fig.height = 5, fig.width = 8----
ggplot(heights_near_top_bot_3,
       aes(x = year,
             y = pred,
             group = country,
             colour = rank)) + 
  geom_line(colour = "orange") + 
  geom_point(aes(y = height_cm)) + 
  facet_wrap(~rank + country,
             labeller = label_glue("Country: {country} \n{rank} RSS"),
             ncol = 3) + 
  theme(legend.position = "none")


## ----eval=FALSE, echo=TRUE----------------------------------------------------
#> # From CRAN
#> install.packages("brolgar")
#> # Development version
#> remotes::install_github("njtierney/brolgar")

